property spriteNum, getPDLError, myCharacters, myCase, myCaseChange, myAllowRETURN, myBeepMode, myAllowFULLSTOP, myMember, myMemberType, myLowercaseAccents, myUPPERCASEAccents

on getBehaviorDescription me
  return "FILTER INPUT CHARACTERS" & RETURN & RETURN & "Enter the appropriate characters in the Behavior Parameters dialog (Don't forget to take the space character into account)." & RETURN & RETURN & "This behavior can also be used to force the case of input characters. " & "It can handle all standard accents... " & "if the font you are using can display them." & RETURN & RETURN & "The option to 'correct case automatically' can be used to override the Caps Lock." & RETURN & RETURN & "You can choose to convert letters automatically to either upper or lower case, or you can accept a mix of upper and lower case characters and limit input to those particular letters in that particular case (select the 'case as entered' option)." & RETURN & RETURN & "PERMITTED MEMBER TYPES:" & RETURN & "Field and Text members" & RETURN & RETURN & "PARAMETERS:" & RETURN & "* Accepted characters" & RETURN & "* Correct case automatically? (TRUE | FALSE)" & RETURN & "* Allow RETURN and ENTER characters?" & RETURN & "* Beep if input character is invalid? (TRUE | FALSE)"
end

on getBehaviorTooltip me
  return "Use with Field and Text members." & RETURN & RETURN & "Limit the characters that the user can type into a Text or Field member. " & "Invalid characters do not appear (a system beep may be provoked instead). " & "Option: automatically convert accepted characters to the chosen case."
end

on resolve prop
  case prop of
    myAllowRETURN:
      choiceslist = ["yes", "no"]
      lookup = [#yes, #no]
    otherwise:
      nothing()
  end case
  return lookup[findPos(choiceslist, prop)]
end

on beginSprite me
  myAllowRETURN = resolve(myAllowRETURN)
  Initialize(me)
end

on keyDown me
  Filter(me)
end

on Initialize me
  myMember = sprite(me.spriteNum).member
  case myAllowRETURN of
    #yes:
      put RETURN & ENTER after myCharacters
  end case
  if myAllowFULLSTOP then
    put "." after myCharacters
  end if
  if the platform contains "Macintosh" then
    myUPPERCASEAccents = [numToChar(193), numToChar(192), numToChar(194), numToChar(196), numToChar(195), numToChar(197), numToChar(199), numToChar(201), numToChar(200), numToChar(202), numToChar(203), numToChar(205), numToChar(204), numToChar(206), numToChar(207), numToChar(209), numToChar(211), numToChar(210), numToChar(212), numToChar(214), numToChar(213), numToChar(218), numToChar(217), numToChar(219), numToChar(220), numToChar(198), numToChar(216), numToChar(159)]
    myLowercaseAccents = [numToChar(225), numToChar(224), numToChar(226), numToChar(228), numToChar(227), numToChar(229), numToChar(231), numToChar(233), numToChar(232), numToChar(234), numToChar(235), numToChar(237), numToChar(236), numToChar(238), numToChar(239), numToChar(241), numToChar(243), numToChar(242), numToChar(244), numToChar(246), numToChar(245), numToChar(250), numToChar(249), numToChar(251), numToChar(252), numToChar(230), numToChar(248), numToChar(255)]
  else
    if the platform contains "Windows" then
      myUPPERCASEAccents = [numToChar(193), numToChar(192), numToChar(194), numToChar(196), numToChar(195), numToChar(197), numToChar(199), numToChar(201), numToChar(200), numToChar(202), numToChar(203), numToChar(205), numToChar(204), numToChar(206), numToChar(207), numToChar(209), numToChar(211), numToChar(210), numToChar(212), numToChar(214), numToChar(213), numToChar(218), numToChar(217), numToChar(219), numToChar(220), numToChar(198), numToChar(216), numToChar(159)]
      myLowercaseAccents = [numToChar(225), numToChar(224), numToChar(226), numToChar(228), numToChar(227), numToChar(229), numToChar(231), numToChar(233), numToChar(232), numToChar(234), numToChar(235), numToChar(237), numToChar(236), numToChar(238), numToChar(239), numToChar(241), numToChar(243), numToChar(242), numToChar(244), numToChar(246), numToChar(245), numToChar(250), numToChar(249), numToChar(251), numToChar(252), numToChar(230), numToChar(248), numToChar(255)]
    end if
  end if
  myCharacters = GetCharacterList(me, myCharacters)
  myMember.editable = 1
end

on Filter me
  if [48, 51, 117, 115, 116, 119, 121, 123, 124, 125, 126].getPos(the keyCode) then
    pass()
  end if
  theKey = the key
  if myAllowFULLSTOP then
    if theKey = "." then
      pass()
    end if
  end if
  if myCharacters.getPos(theKey) then
    pass()
  else
    if myCaseChange then
      theKey = ChangeCase(me, theKey)
      if myCharacters.getPos(theKey) then
        Insert(me, theKey)
        exit
      end if
    end if
    if myBeepMode then
      beep()
    end if
    dontPassEvent()
  end if
end

on Insert me, theKey
  startChar = the selStart
  if startChar = the selEnd then
    if myMemberType = #field then
      put theKey after char startChar of field myMember
    else
      theText = myMember.text
      put theKey after char startChar of startChar
      myMember.text = theText
    end if
  else
    myMember.char[startChar + 1..the selEnd] = theKey
  end if
  set the selEnd to startChar + 1
  set the selStart to startChar + 1
  dontPassEvent()
end

on GetCharacterList me, theString
  characterList = []
  charCount = theString.char.count
  repeat with i = 1 to charCount
    theChar = theString.char[1]
    delete me.char[1]
    if characterList.getPos(theChar) then
      next repeat
    end if
    case myCase of
      "UPPERCASE and lowercase":
        otherCase = ChangeCase(me, theChar)
        if (otherCase < theChar) or (otherCase > theChar) then
          if not characterList.getPos(otherCase) then
            characterList.append(otherCase)
          end if
        end if
      "UPPERCASE ONLY":
        theChar = ChangeCase(me, theChar, #upper)
      "lowercase only":
        theChar = ChangeCase(me, theChar, #lower)
    end case
    characterList.append(theChar)
  end repeat
  return characterList
end

on ChangeCase me, theChar, lowerOrUPPER
  asciiCode = charToNum(theChar)
  case lowerOrUPPER of
    #lower, #upper:
    otherwise:
      if asciiCode > 90 then
        lowerOrUPPER = #upper
      else
        lowerOrUPPER = #lower
      end if
  end case
  case lowerOrUPPER of
    #upper:
      accent = myLowercaseAccents.getPos(theChar)
      if accent then
        theChar = myUPPERCASEAccents[accent]
      else
        if (asciiCode > 96) and (asciiCode < 123) then
          theChar = numToChar(asciiCode - 32)
        end if
      end if
    #lower:
      accent = myUPPERCASEAccents.getPos(theChar)
      if accent then
        theChar = myLowercaseAccents[accent]
      else
        if (asciiCode > 64) and (asciiCode < 91) then
          theChar = numToChar(asciiCode + 32)
        end if
      end if
  end case
  return theChar
end

on isOKToAttach me, aSpriteType, aSpriteNum
  case aSpriteType of
    #graphic:
      return getPos([#field, #text], sprite(aSpriteNum).member.type) <> 0
    #script:
      return 0
  end case
end

on getPropertyDescriptionList me
  if not (the currentSpriteNum) then
    exit
  end if
  return [#myCharacters: [#comment: "Accepted characters", #format: #string, #default: "abcdefghijklmnopqrstuvwxyz 0123456789"], #myCase: [#comment: EMPTY, #format: #string, #range: ["UPPERCASE and lowercase", "UPPERCASE ONLY", "lowercase only", "case as entered above"], #default: "UPPERCASE and lowercase"], #myCaseChange: [#comment: "Correct case automatically?", #format: #boolean, #default: 1], #myAllowRETURN: [#comment: "Allow RETURN and ENTER characters?", #format: #string, #range: ["yes", "no"], #default: "yes"], #myAllowFULLSTOP: [#comment: "Allow FULL STOP character?", #format: #boolean, #default: 1], #myBeepMode: [#comment: "Beep if input character is invalid?", #format: #boolean, #default: 1]]
end
